import csv
import random


class DailyPrompt:
    # initialize the object and load the prompts from a CSV file.
    def __init__(self, prompts_file):
        self.prompts_file = prompts_file
        self.prompts = []
        with open(prompts_file, 'r') as f:
            reader = csv.reader(f)
            for row in reader:
                self.prompts.append(row[0])

    def get_prompt(self):
        """
        This function randomly selects a prompt
        from the list of prompts and return it to the user.
        Args:
            self
        Returns:
            prompt (data)
        """
        prompt = random.choice(self.prompts)
        return str(prompt)

    def get_prompt_easy(self):
        """
        This function randomly selects an EASY/SHORT prompt
        from the list of prompts and return it to the user.
        Args:
            self
        Returns:
            prompt (data)
        Note: an easy prompt is defined here as
        a prompt with less than 11 words
        """
        easy_prompt = random.choice(self.prompts)
        if len(easy_prompt.split()) < 11:
            return str(easy_prompt)
        else:
            return None

    def add_prompt(self, prompt):
        """
        This function adds a new prompt to the list of prompts.
        Args:
            Prompt to be appended
        Returns:
            None: see note
        Note: appends the given prompt to the csv file of prompts
        """
        self.prompts.append(prompt)

    def remove_prompt(self, prompt):
        """
        This function removes a prompt from the list of prompts.
        Args:
            Prompt to be removed
        Returns:
            None: see note
        Note: removes the given prompt from the list of prompts
        """
        self.prompts.remove(prompt)

    def search_prompt(self, keyword):
        """
        This function searches the list of prompts for a keyword
        and returns a list of prompts that contain the keyword.
        Args:
            keyword: word to search for
        Returns:
            result: list of prompts containing the keyword
        """
        result = []
        for prompt in self.prompts:
            if keyword.lower() in prompt.lower():
                result.append(prompt)
        return result



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
import sphinx_rtd_theme, sys, os
from recommonmark.transform import AutoStructify

project = 'journalyze'
copyright = '2023, Johana De La Rosa'
author = 'Johana De La Rosa'
release = '0.1.0'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ['recommonmark', 'sphinx.ext.autodoc', 'sphinx.ext.napoleon']
source_suffix = ['.rst', '.md']

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
html_static_path = ['_static']

def setup(app):
    app.add_config_value('recommonmark_config', {
            'auto_toc_tree_section': 'Contents',
            }, True)
    app.add_transform(AutoStructify)

sys.path.insert(0, os.path.abspath('..'))



from .journalyze import DailyPrompt

if __name__ == "__main__":
    dp = DailyPrompt('journalyze/prompts.csv')
    prompt = dp.get_prompt()
    print(prompt)

    dp.add_prompt('What was something you learned today?')
    dp.remove_prompt('What was your favorite memory from last year?')



from journalyze import *
from unittest.mock import patch
import unittest
import csv
from io import StringIO
from unittest.mock import patch


class TestDailyPrompt(unittest.TestCase):
    def setUp(self):
        self.prompts_file = 'test_prompts.csv'
        self.prompts = [
            'What was your favorite part of today?',
            'What are you grateful for today?',
            'What is your favorite book?',
            'Write about a time when you overcame a fear.',
            'Write about a time when you felt proud of yourself.',
        ]
        with open(self.prompts_file, 'w', newline='') as f:
            writer = csv.writer(f)
            for prompt in self.prompts:
                writer.writerow([prompt])
        self.dp = DailyPrompt(self.prompts_file)

    """def test_init(self):
        self.assertEqual(self.dp.prompts, self.prompts)
    """

    def test_get_prompt(self):
        prompt = self.dp.get_prompt()
        self.assertIn(prompt, self.prompts)

    def test_get_prompt_easy(self):
        prompt = self.dp.get_prompt_easy()
        assert isinstance(prompt, str)
        assert len(prompt.split()) < 11

    def test_add_prompt(self):
        new_prompt = 'What is something you learned today?'
        self.dp.add_prompt(new_prompt)
        self.assertIn(new_prompt, self.dp.prompts)

    def test_remove_prompt(self):
        prompt_to_remove = 'What was your favorite part of today?'
        self.dp.remove_prompt(prompt_to_remove)
        self.assertNotIn(prompt_to_remove, self.dp.prompts)

    def test_search_prompt(self):
        prompts_file = "test_prompts.csv"
        daily_prompt = DailyPrompt(prompts_file)

        # Test search for "time"
        expected_result = [
            'Write about a time when you overcame a fear.',
            'Write about a time when you felt proud of yourself.',
        ]
        self.assertEqual(daily_prompt.search_prompt("time"), expected_result)

        # Test search for "favorite"
        expected_result = ['What was your favorite part of today?', 'What is your favorite book?']
        self.assertEqual(daily_prompt.search_prompt("favorite"), expected_result)

        # Test search for "dog" (not present in prompts)
        expected_result = []
        self.assertEqual(daily_prompt.search_prompt("dog"), expected_result)


class TestDailyPromptIntegration(unittest.TestCase):
    def setUp(self):
        self.prompts_file = 'test_prompts.csv'
        self.prompts = ['What was your favorite part of today?', 'What are you grateful for today?']
        with open(self.prompts_file, 'w', newline='') as f:
            writer = csv.writer(f)
            for prompt in self.prompts:
                writer.writerow([prompt])
        self.dp = DailyPrompt(self.prompts_file)

    def test_integration(self):
        # Check that the initial prompts are in the list
        self.assertEqual(self.dp.prompts, self.prompts)

        # Check that a prompt can be retrieved
        prompt = self.dp.get_prompt()
        self.assertIn(prompt, self.prompts)

        # Check that a new prompt can be added
        new_prompt = 'What is something you learned today?'
        self.dp.add_prompt(new_prompt)
        self.assertIn(new_prompt, self.dp.prompts)

        # Check that a prompt can be removed
        prompt_to_remove = 'What was your favorite part of today?'
        self.dp.remove_prompt(prompt_to_remove)
        self.assertNotIn(prompt_to_remove, self.dp.prompts)


if __name__ == '__main__':
    unittest.main()
